#include <msp430.h>
#include "serial.h"
#include "utils.h"
#include "ir.h"

/*
 * main.c
 */

void tohex(int n,char *buff, unsigned int count)
{
	if (n<0) n=-n;
	unsigned int i;
	for(i=count;i!=0;--i)
	{
		unsigned int x=n%16;
		if (x<=9) buff[i-1]=x+'0';
		else buff[i-1]=x-10+'A';
		n/=16;
	}
	buff[count]='\0';
}

void tostr(int n,char *buff, unsigned int count)
{
	if (n<0) n=-n;
	unsigned int i;
	for(i=count;i!=0;--i)
	{
		buff[i-1]=(n%10)+'0';
		n/=10;
	}
	buff[count]='\0';
}

void setclk()
{
    //BCSCTL1 |= DIVS_0 + DIVM_0 + DIVA_0 + RSEL2 + RSEL1 + RSEL0; //DIVA /1, max RSEL
	//DCOCTL=DCO1+DCO0+DCO2; //max DCO

	BCSCTL1 |= DIVS_0 + DIVM_0 + DIVA_0; //DIVA,S,M /1, RSEL: default
	//BCSCTL1 |= DIVS_0 + DIVM_0 + DIVA_3; //DIVA,S,M /1, RSEL: default
	BCSCTL1 &= ~XTS; //LFXTCLK Low Frequency

    BCSCTL1 &= ~XT2OFF; //power on XT2
    do
    {
    	IFG1 &= ~OFIFG; //clear oscillator fault
    	unsigned int i;
    	for (i = 0xFF; i != 0; i--);
    }
    while ((IFG1 & OFIFG)); //oscillator fault

    BCSCTL2 |= SELM_2; //MCLK: XT2CLK
    BCSCTL2 |= SELS; //SMCLK: XT2CLK
}

void init() {
	utils_killdog();
	setclk();

	//pull down IO ports
	P1DIR=0xFF;
	P1OUT=0x00;
	P2DIR=0xFF;
	P2OUT=0x00;
	P3DIR=0xFF;
	P3OUT=0x00;
	P4DIR=0xFF;
	P4OUT=0x00;
	P5DIR=0xFF;
	P5OUT=0x00;
	P6DIR=0xFF;
	P6OUT=0x00;
}

void ir_read_test() {
	ir_init();
	while(1) {
		/*while(IR_DATA);
		unsigned long l=0,h=0;
		while(!IR_DATA) {
			++l;
			utils_delay_us(100);
		}
		while(IR_DATA) {
			++h;
			utils_delay_us(100);
		}
		char buff[10];
		tostr(l,buff,7);
		serial_write_str("Low: ");
		serial_write_str(buff);
		tostr(h,buff,7);
		serial_write_str(", High: ");
		serial_write_str(buff);
		serial_write_str("\r\n");
		utils_delay_ms(1000);*/
		unsigned char buff[20];
		int count=0;
		count=ir_read(buff, 20);
		if(count>=0) {
			char buff1[10];
			tostr(count,buff1,7);
			serial_write_str("Count: ");
			serial_write_str(buff1);
			serial_write_str("\r\n");
				unsigned int i;
			for(i=0;i!=count;++i) {
				tohex(buff[i],buff1,2);
				serial_write_str(buff1);
				serial_write_str(" ");
			}
			serial_write_str("\r\n");
		} else {
			switch(count) {
			case -1:
				serial_write_str("Head error\r\n");
				break;
			default:
				serial_write_str("Unknown error\r\n");
				break;
			}
		}
	}
}

void ir_send_test() {
	unsigned char data[]={0x04, 0xDB, 0x82, 0x7D};
	while(1) {
		ir_nec_send(data, 4);
		P2OUT=~BIT7;
		utils_delay_ms(100);
		P2OUT=0xFF;
		utils_delay_ms(1000);
		//ir_tick();
	}
}

int main(void) {
	init();
	serial_init();
	serial_write_str("hello\r\n");
	//ir_read_test();
	ir_send_test();

	return 0;
}
